package at.jku.ssw.pi.sort.impl;

import java.util.Comparator;

import at.jku.ssw.pi.sort.Sorter;

/** A fast sorting algorithm developed by Tony Hoare. */
@SuppressWarnings({ "rawtypes", "unchecked" })
public class QuickSort extends Sorter {
	private static final boolean DEBUGGING = false;

	/** Sorts the array <code>items</code> of comparable objects. */
	public Object[] sort(Object[] data, Comparator cmp) {
		userArray = data;
		userCmp = cmp;
		if (userArray.length <= 0 || isAlreadySorted())
			return data;
		quicksort(0, userArray.length - 1);
		if (isAlreadySorted())
			return data;
		if (DEBUGGING) {
			// debug ensure sort is working
			if (!isAlreadySorted())
				System.out.println("Sort failed");
		}
		return data;
	} // end sort
		// recursive quicksort that breaks array up into sub
		// arrays and sorts each of them.

	private void quicksort(int p, int r) {
		if (p < r) {
			int q = partition(p, r);
			if (q == r) {
				q--;
			}
			quicksort(p, q);
			quicksort(q + 1, r);
		} // end if
	} // end quicksort
		// Partition by splitting this chunk to sort in two and
		// get all big elements on one side of the pivot and all
		// the small elements on the other.

	private int partition(int lo, int hi) {
		Object pivot = userArray[lo];
		while (true) {
			while (userCmp.compare(userArray[hi], pivot) >= 0 && lo < hi) {
				hi--;
			}
			while (userCmp.compare(userArray[lo], pivot) < 0 && lo < hi) {
				lo++;
			}
			if (lo < hi) {
				// exchange objects on either side of the pivot
				Object T = userArray[lo];
				userArray[lo] = userArray[hi];
				userArray[hi] = T;
			} else
				return hi;
		} // end while
	} // end partition
		// check if user's array is already sorted

	private boolean isAlreadySorted() {
		for (int i = 1; i < userArray.length; i++) {
			if (userCmp.compare(userArray[i], userArray[i - 1]) < 0)
				return false;
		}
		return true;
	} // end isAlreadySorted

	private Object[] userArray;
	private Comparator userCmp;
} // end class QuickSort
