package at.jku.ssw.pi.sort.ue11;

import java.util.Arrays;
import java.util.Comparator;

import at.jku.ssw.pi.Out;
import at.jku.ssw.pi.sort.Searcher;
import at.jku.ssw.pi.sort.Sorter;
import at.jku.ssw.pi.sort.impl.BubbleSort;
import at.jku.ssw.pi.sort.impl.InsertionSort;
import at.jku.ssw.pi.sort.impl.JavaUtilArraysSort;
import at.jku.ssw.pi.sort.impl.QuickSort;
import at.jku.ssw.pi.sort.impl.SelectionSort;
import at.jku.ssw.pi.util.Measurement;
import at.jku.ssw.pi.util.SorterEvaluationUI;
import at.jku.ssw.pi.util.Timer;

@SuppressWarnings({ "rawtypes" })
public class SortTest {
	private static class IntegerComparator implements Comparator<Integer> {
		@Override
		public int compare(Integer o1, Integer o2) {
			return o1 - o2;
		}
	}

	public static void main(String[] args) {
		test1();
		test2();
		test3();
		test4();
	}

	static void test1() {
		Sorter heapSort = new SimpleHeapSort();
		Object[] values = new Integer[] { 10, 3, 1, 7, 6, 5, 4, 2, 9, 8 };
		values = heapSort.sort(values, new IntegerComparator());
		for (int i = 0; i < values.length; i++) {
			Out.print(" " + values[i]);
		}
		Out.println();
	}

	static void test2() {
		Sorter heapSort = new MergeSort();
		Object[] values = new Integer[] { 10, 3, 1, 7, 6, 5, 4, 2, 9, 8 };
		values = heapSort.sort(values, new IntegerComparator());
		for (int i = 0; i < values.length; i++) {
			Out.print(" " + values[i]);
		}
		Out.println();
	}

	static void test3() {
		Comparator cmp = new IntegerComparator();
		Searcher binarySearch = new BinarySearch();
		Object[] values = new Integer[] { 4, 8, 15, 16, 23, 42, 108, 316, 815, 7418880 };
		for (int i = 0; i < values.length; i++) {
			Out.println(" Found " + values[i] + " at: " + binarySearch.search(values, values[i], cmp));
		}
		Out.println(" Found 0 at:   " + binarySearch.search(values, 0, cmp));
		Out.println(" Found 9 at:   " + binarySearch.search(values, 9, cmp));
		Out.println(" Found 407 at: " + binarySearch.search(values, 407, cmp));
		Out.println(" Found 8417880 at: " + binarySearch.search(values, 8417880, cmp));
		Out.println();
	}

	static void test4() {
		Comparable[] data = generateIntegers(10000, Order.RANDOM);

		Sorter[] sorters = new Sorter[] { new BubbleSort(), new SelectionSort(), new InsertionSort(), new MergeSort(),
				new JavaUtilArraysSort(), new QuickSort(), new SimpleHeapSort() };

		Measurement[] measurements = new Measurement[sorters.length];
		for (int i = 0; i < sorters.length; i++) {
			measurements[i] = new Measurement(sorters[i], data);
		}

		final int WARMUP = 50;
		for (int i = 0; i < sorters.length; i++) {
			repeatMeasurements(measurements[i], WARMUP);
		}

		final int REPEAT = 10;
		for (int i = 0; i < measurements.length; i++) {
			repeatMeasurements(measurements[i], REPEAT);
		}
		Arrays.sort(measurements);

		new SorterEvaluationUI(measurements, data).show();
	}

	private static void repeatMeasurements(Measurement measurement, int repeat) {
		measurement.timer = Timer.start();
		for (int i = 0; i < repeat; i++) {
			measurement.timer.pause();
			Object[] data = measurement.unsortedData.clone();
			measurement.timer.continu();
			measurement.sortedData = measurement.sorter.sort(data, new IntegerComparator());
		}
		measurement.timer.stop();
	}

	public static enum Order {

		ASCENDING, DESCENDING, RANDOM
	};

	private static Comparable[] generateIntegers(int cnt, Order order) {
		Integer[] arr = new Integer[cnt];
		for (int i = 0; i < cnt; i++) {
			switch (order) {
			case ASCENDING:
				arr[i] = i;
				break;
			case DESCENDING:
				arr[i] = cnt - i;
				break;
			case RANDOM:
				arr[i] = (int) (Math.random() * cnt);
				break;
			}
		}
		return arr;
	}

}
